// (C) Copyright 2009, Jun Zhu (junzhu [at] cs [dot] cmu [dot] edu)

// This file is part of MedLDA.

// MedLDA is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.

// MedLDA is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA

#include "defs.h"
#include "corpus.h"
#include "cokus.h"
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <iostream>

using namespace std;

Corpus::Corpus(void)
{
	docs = NULL;
}

Corpus::~Corpus(void)
{
	if (docs) 
	{
		for(int d = 0; d < num_docs; ++d)
		{
			Document& dc = docs[d];
			if(dc.words) delete[] dc.words; 
			dc.words = NULL;
		}
		delete[] docs;
	}
}

void Corpus::read_data(char* data_filename)
{
	if (!data_filename)
	{
		printf("ERROR> data_file is empty error!\n");	
		exit(1);
	}
	FILE *fileptr;
	int length, word, n, nd, nw, label;
	nw = 0;
	num_terms = 0;

	printf("reading data from %s\n", data_filename);
	fileptr = fopen(data_filename, "r");
	if (!fileptr) 
	{
		printf("ERROR> data_file is empty error!\n");	
		exit(1);
	}

	fscanf(fileptr, "%d", &num_docs);
	docs = new Document[num_docs];
	for (nd = 0; nd < num_docs; ++nd)
	{
		fscanf(fileptr, "%d", &length);
		Document& dc = docs[nd];
		dc.length = length;
		dc.words = new int[length];
		fscanf(fileptr, "%d", &label);
		dc.gndlabel = label - OFFSET_LABEL;
		int* wptr = dc.words;
		for (n = 0; n < length; n++) 
		{
			fscanf(fileptr, "%d", &word);
			word -= OFFSET;
			*wptr++ = word;
			if (word >= nw) { nw = word + 1; }
		}
	}
	fclose(fileptr);
	num_terms = nw;
}

void Corpus::read_data(char* data_filename, const int OOV)
{
	if (!data_filename)
	{
		printf("ERROR> data_file is empty error!\n");	
		exit(1);
	}
	FILE *fileptr;
	int length, word, n, nd, label;
	num_terms = OOV;

	printf("reading data from %s\n", data_filename);
	fileptr = fopen(data_filename, "r");
	if (!fileptr) {
		printf("ERROR> data_file is empty error!\n");	
		exit(1);
	}

	fscanf(fileptr, "%d", &num_docs);
	docs = new Document[num_docs];
	for (nd = 0; nd < num_docs; ++nd) {
		fscanf(fileptr, "%d", &length);
		Document& dc = docs[nd];
		dc.length = length;
		dc.words = new int[length];
		fscanf(fileptr, "%d", &label);
		dc.gndlabel = label - OFFSET_LABEL;
		int* wptr = dc.words;
		for (n = 0; n < length; n++) {
			fscanf(fileptr, "%d", &word);
			word -= OFFSET;
			if (word >= OOV) word = (int)(myrand() * OOV);
			*wptr++ = word; 
		}
	}
	fclose(fileptr);
}

void Corpus::print(char* o_file)
{
	FILE* fout = fopen(o_file, "w");
	if (!fout)
	{
		printf("ERROR> Cannot open output file!\n");	
		return;
	}
	fprintf(fout, "%d\n", num_docs);
	for(int i = 0; i < num_docs; ++i)
	{
		Document& d = docs[i];
		fprintf(fout, "%d %d", d.length, d.gndlabel);
		for(int j = 0; j < d.length; ++j)
			fprintf(fout, " %d", d.words[j]);
		fprintf(fout, "\n");
	}
	fclose(fout);
}

void Corpus::shuffle()
{
	srand(time(NULL));
	for (int n = 0; n < num_docs * 100; n++)
	{
		int ix1 = rand() % num_docs;
		int ix2 = rand() % num_docs;
		if ( ix1 == ix2 ) continue;
													
		Document p = docs[ix1];
		docs[ix1] = docs[ix2];
		docs[ix2] = p;
	}
}

Corpus* Corpus::get_traindata(const int nfold, const int foldix)
{
	int nunit 	= num_docs / nfold;
	int D			= num_docs - nunit;

	Corpus *subc = new Corpus();
	subc->docs = new Document[D];
	subc->num_docs = D;
	subc->num_terms = 0;

	int *ndcw;
	int *dcw;
	int nd = 0, nw = 0, n;
	for ( int d = 0; d < num_docs; d++ )
	{
		if ( (d >= foldix*nunit) && ( d < (foldix+1)*nunit ) ) 
				continue;

		Document &dc 	= docs[d];
		Document &ndc 	= subc->docs[nd];
		ndc.length = dc.length;
		ndc.words = new int[ndc.length];
		ndc.gndlabel = dc.gndlabel; 
		n 		= ndc.length;
		ndcw 	= ndc.words;
		dcw	= dc.words;
		while(n--)
		{
			if (*dcw >= nw) nw = *dcw + 1;
			*ndcw++ = *dcw++;
		}
		nd++;
	}
	subc->num_terms = nw;
	return subc;
}

Corpus* Corpus::get_testdata(const int nfold, const int foldix)
{
	int nunit 	= num_docs / nfold;
	Corpus *subc = new Corpus();
	subc->docs = new Document[nunit];
	subc->num_docs = nunit;
	subc->num_terms = 0;

	int *ndcw;
	int *dcw;
	int nw = 0, n;
	for (int nd = 0; nd < nunit; ++nd)
	{
		int d = foldix * nunit + nd;
		Document &dc 	= docs[d];
		Document &ndc 	= subc->docs[nd];
		ndc.length = dc.length;
		ndc.words = new int[ndc.length];
		ndc.gndlabel = dc.gndlabel; 
		n 		= ndc.length;
		ndcw 	= ndc.words;
		dcw	= dc.words;
		while(n--)
		{
			if (*dcw >= nw) nw = *dcw + 1;
			*ndcw++ = *dcw++;
		}
	}
	subc->num_terms = nw;
	return subc;
}
