#include "defs.h"
#include "utils.h"
#include "cokus.h"
#include "gibbsMedLDA.h"
#include "params.h"

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>

using namespace std;

double inner_cv_C(Corpus* corpus, Params* param);
double inner_cv_alpha(Corpus* corpus, Params* param);

int main(int argc, char** argv)
{
	seedMT( time(NULL) );

	if (argc > 1)
	{
		Params param;
		param.read_settings("settings.txt");
		param.NTOPICS = atoi(argv[2]);
		param.NLABELS = atoi(argv[3]);
		param.INITIAL_ALPHA = atof(argv[4]);
		param.SAMPLING_LAG = atoi(argv[5]);
		param.INITIAL_C = atof(argv[6]);
		param.EM_MAX_ITER = atoi(argv[7]);
		param.NFOLDS  = atoi(argv[8]);
		printf("K: %d, Alpha: %.3f, C: %.3f, svm: %d, SampleLag: %d\n", param.NTOPICS, param.INITIAL_ALPHA, param.INITIAL_C, param.SVM_ALGTYPE, param.SAMPLING_LAG);
		sprintf(param.dir, "GML-K%d-A%d-L%d-C%d-F%d", param.NTOPICS, (int)param.INITIAL_ALPHA,(int)param.SAMPLING_LAG,  (int)param.INITIAL_C, param.NFOLDS);
		make_directory(param.dir);
		sprintf(param.model_filename, "%s/model-final.txt", param.dir);

		if ( strcmp(argv[1], "-estinf") == 0 ) 
		{
			strcpy(param.train_filename, argv[9]);
			strcpy(param.test_filename, argv[10]);

			if (param.INNER_CV_C)
			{
				Corpus* corpus = new Corpus();
				corpus->read_data(param.train_filename);
				corpus->shuffle();
				param.INITIAL_C = inner_cv_C(corpus, &param);
				printf("\n\nBest C: %5.10f\n\n", param.INITIAL_C);
				delete corpus;
			}
			if (param.INNER_CV_ALPHA)
			{
				Corpus* corpus = new Corpus();
				corpus->read_data(param.train_filename);
				corpus->shuffle();
				param.INITIAL_ALPHA = inner_cv_alpha(corpus, &param);
				printf("\n\nBest ALPHA: %5.10f\n\n", param.INITIAL_ALPHA);
				delete corpus;
			}

			GibbsMedLDA model(&param);
			model.init_est();
			model.run_em();
			model.save_model(param.model_filename);

			Corpus *test_corpus = new Corpus();
			// Read data and remove unseen words
			test_corpus->read_data(param.test_filename, model.m_nV);
			model.init_inf(test_corpus);
			double tAvgEnt = 0;
			double dAcc = model.inference(&tAvgEnt);
			model.save_results(tAvgEnt);
			printf("Accuracy: %5.10f\n", dAcc);
			printf("testAverageEntropy: %5.10f\n", tAvgEnt);
		}
		else if ( strcmp(argv[1], "-est") == 0 ) 
		{
			strcpy(param.train_filename, argv[8]);

			if (param.INNER_CV_C)
			{
				Corpus* corpus = new Corpus();
				corpus->read_data(param.train_filename);
				corpus->shuffle();
				param.INITIAL_C = inner_cv_C(corpus, &param);
				printf("\n\nBest C: %5.10f\n\n", param.INITIAL_C);
				delete corpus;
			}

			if (param.INNER_CV_ALPHA)
			{
				Corpus* corpus = new Corpus();
				corpus->read_data(param.train_filename);
				corpus->shuffle();
				param.INITIAL_ALPHA = inner_cv_alpha(corpus, &param);
				printf("\n\nBest ALPHA: %5.10f\n\n", param.INITIAL_ALPHA);
				delete corpus;
			}

			GibbsMedLDA model(&param);
			model.init_est();
			model.run_em();
			model.save_model(param.model_filename);
		}
		else if ( strcmp(argv[1], "-estc") == 0 ) 
		{
			GibbsMedLDA model(&param);
			model.init_estc(argv[7]);
			model.run_em();
			model.save_model(param.model_filename);
		}
		else if ( strcmp(argv[1], "-inf") == 0 ) 
		{
			strcpy(param.test_filename, argv[9]);
			GibbsMedLDA model(&param);
			printf("Model file: %s\n", param.model_filename);
			model.init_inf(param.model_filename);
			double tAvgEnt = 0;
			double dAcc = model.inference(&tAvgEnt);
			model.save_results(tAvgEnt);
			printf("Accuracy: %5.10f\n", dAcc);
		}
		else
		{
			printf("Usages: medlda -estinf [K] [lables] [Alpha] [SampleLag] [C] [MAX-EM-ITER] [folds] [training_data] [test_data]\n");
			//printf("        medlda -est [K] [lables] [Alpha] [SampleLag] [C] [folds] [training_data]\n");
			//printf("        medlda -estc [K] [lables] [Alpha] [SampleLag] [C] [folds] [prev_state_file]\n");
			//printf("        medlda -inf [K] [lables] [Alpha] [SampleLag] [C] [folds] [test_data]\n");
		}
	}
	else
	{
		//printf("Usages: medlda -estinf [K] [lables] [Alpha] [SampleLag] [C] [folds] [training_data] [test_data]\n");
		printf("Usages: medlda -estinf [K] [lables] [Alpha] [SampleLag] [C] [MAX-EM-ITER] [folds] [training_data] [test_data]\n");
		//printf("        medlda -est [K] [lables] [Alpha] [SampleLag] [C] [folds] [training_data]\n");
		//printf("        medlda -estc [K] [lables] [Alpha] [SampleLag] [C] [folds] [prev_state_file]\n");
		//printf("        medlda -inf [K] [lables] [Alpha] [SampleLag] [C] [folds] [test_data]\n");
	}
	return 0;
}

double inner_cv_C(Corpus* corpus,  Params* param)
{
	Params* inner_param = param->copy();
	sprintf(inner_param->dir, "%s/innerCVC", param->dir);
	make_directory(inner_param->dir);

	double dBestC = 0.0;
	double dBestAccuracy = 0.0;
	double dAcc = 0;
	double tAvgEnt = 0;
	for( int k = 0; k < param->CV_PARAMNUM_C; ++k)
	{
		printf("\n\n$$$ Learning with C: %.4f $$$ \n\n", param->cv_param_vec_C[k]);
		inner_param->INITIAL_C = param->cv_param_vec_C[k];
		double dAccuracy = 0;
		for (int i = 0; i < param->INNER_FOLDNUM_C; ++i)
		{
			printf("\n FOLD: %d \n\n", i);
			Corpus* trn_corpus = corpus->get_traindata(inner_param->INNER_FOLDNUM_C, i); 	
			Corpus* tst_corpus = corpus->get_testdata(inner_param->INNER_FOLDNUM_C, i); 	

			GibbsMedLDA* gml = new GibbsMedLDA(inner_param);
			gml->init_est(trn_corpus);
			gml->run_em();
			gml->init_inf(tst_corpus);
			dAcc = gml->inference(&tAvgEnt);
			dAccuracy += dAcc / param->INNER_FOLDNUM_C;
			delete gml;
		}
		printf("@@@ Avg Accuracy: %.4f\n\n", dAccuracy);
		if (dAccuracy > dBestAccuracy)
		{
			dBestAccuracy = dAccuracy;
			dBestC		  = inner_param->INITIAL_C;
		}
	}
	delete inner_param;
	return dBestC;
}

double inner_cv_alpha(Corpus* corpus, Params* param)
{
	char buf[512];
	Params* inner_param = param->copy();
	sprintf(inner_param->dir, "%s/innerCVAlpha", param->dir);
	sprintf(inner_param->model_filename, "%s/model-final.txt", inner_param->dir);
	make_directory(inner_param->dir);

	double dBestALPHA 	= 0.0;
	double dBestAccuracy = 0.0;
	double dAcc 			= 0;
	double tAvgEnt = 0;
	for( int k = 0; k < param->CV_PARAMNUM_ALPHA; ++k)
	{
		printf("\n\n$$$ Learning with ALPHA: %.4f $$$ \n\n", param->cv_param_vec_ALPHA[k]);
		inner_param->INITIAL_ALPHA = param->cv_param_vec_ALPHA[k];
		double dAccuracy = 0;
		for (int i = 0; i < param->INNER_FOLDNUM_ALPHA; ++i)
		{
			printf("\n FOLD: %d \n\n", i);
			Corpus* trn_corpus = corpus->get_traindata(inner_param->INNER_FOLDNUM_ALPHA, i); 	
			Corpus* tst_corpus = corpus->get_testdata(inner_param->INNER_FOLDNUM_ALPHA, i); 	

			GibbsMedLDA* gml = new GibbsMedLDA(inner_param);
			gml->init_est(trn_corpus);
			gml->run_em();
			gml->init_inf(tst_corpus);
			dAcc = gml->inference(&tAvgEnt);
			printf("[K: %d; Alpha: %f; Fold: %d] dAcc: %.5f\n", inner_param->NTOPICS, inner_param->INITIAL_ALPHA, i, dAcc);
			dAccuracy += dAcc / param->INNER_FOLDNUM_ALPHA;
			delete gml;
		}
		printf("@@@ [K: %d; Alpha: %f] Avg_Acc: %.5f\n\n", inner_param->NTOPICS, inner_param->INITIAL_ALPHA, dAccuracy);
		if (dAccuracy > dBestAccuracy)
		{
			dBestAccuracy = dAccuracy;
			dBestALPHA	  = inner_param->INITIAL_ALPHA;
		}
	}
	delete inner_param;
	return dBestALPHA;
}
